require_relative 'path_loader'

describe 'YubiRuby::FakeYubikey' do
  @@UID = 'ffa4f1fa55e0'
  @@KEY = '884b1fc18da3888b59460f6e33acabf5'
  
  before :all do
    @key = YubiRuby::FakeYubikey.new
  end
  
  it "should start in a valid state" do
    @key.uid.should == "000000000000"
    @key.session_use.should == 0
    @key.session_counter.should == 1
    @key.public_id.should == ""
  end
  
  it "should be serializable" do
    @key.public_id = 'hehkhbhv'
    @key.key = @@KEY
    @key.uid = @@UID
    3.times { @key.init }
    File.open('key.dump', 'w+') do |f|  
      Marshal.dump(@key, f)  
    end
    
    key2 = nil
    File.open('key.dump') do |f|  
      key2 = Marshal.load(f)  
    end
    
    key2.public_id.should == @key.public_id
    key2.uid.should == @key.uid
    key2.data.session_counter.should == @key.session_counter+1
    key2.key.should == @key.key
  end
  
  describe "#set_random_uid" do
    it "should generate a hex valid string" do
      @key.set_random_uid
      @key.uid.hex?.should == true
    end
  end
  
  describe "#set_random_key" do
    it "should generate a hex valid string" do
      @key.set_random_key
      @key.key.hex?.should == true
    end
  end
  
  describe '#otp' do
    it "should generate valid otp" do
      aes = @key.set_random_key
      @key.set_random_uid
      
      validator = YubiRuby::Yubikey.new aes
      validator.parse(@key.otp).should == true
      validator.uid.should == @key.uid
      validator.session_use.should == @key.session_use
      validator.session_counter.should == @key.session_counter
    end
    
    it "should produce a session overflow after 2**8 generations" do
      @key = YubiRuby::FakeYubikey.new
      aes = @key.set_random_key
      uid = @key.set_random_uid
      session = @key.session_counter
      1.upto(2**8-1) do |i|
        @key.otp 
        @key.session_counter.should == session
        @key.session_use.should == i
      end
      @key.otp
      @key.session_counter.should == session+1
      @key.session_use.should == 0
    end
    
    it "should raise YubiRuby::YubikeySessionCounterOverflow after SESSION_COUNTER_OVERFLOW initialization" do
      key = YubiRuby::FakeYubikey.new
      aes = key.set_random_key
      uid = key.set_random_uid
      (session = key.session_counter).should == 1
      2.upto(YubiRuby::FakeYubikey::Data::SESSION_COUNTER_OVERFLOW) do |i|
        lambda { key.init }.should_not raise_error(YubiRuby::YubikeySessionCounterOverflow)
      end
      lambda { key.init }.should raise_error(YubiRuby::YubikeySessionCounterOverflow)
    end
    
    it "should starts with public_id" do
      @key = YubiRuby::FakeYubikey.new 
      @key.key = @@KEY
      @key.uid = @@UID
      @key.public_id = 'hehkhbhv'
      @key.otp[0,8].should == 'hehkhbhv'
    end
  end
  
  describe '#generate_specific_otp_and_fake_yubikey' do
    it "should work ;)" do
      aes = 'df5b9d7591b0b3c2819f7a0dd7d1547d'
      session = 53
      tsl = 31716
      tsh = 106
      counter = 0
      random = 30172
      crc = 58822
      uid = "6d8dec078841"
      check_otp = 'rfechkcdkjrflgrjlirdgivnedbbtcik'
      result = YubiRuby::FakeYubikey.generate_specific_otp_and_fake_yubikey(
        aes, uid, session, tsl, tsh, counter, random)
      mycrc = YubiRuby::CRC16.calculate(result[1].to_s[0...-2]).ones_complement 16

      lambda {
        YubiRuby::Yubikey.new(aes).parse(result[0]).should == true
      }.should_not raise_error(TypeError)
      result[1].key.should == aes
      result[1].session_counter.should == session
      result[1].timestamp_low.should == tsl
      result[1].timestamp_high.should == tsh
      result[1].session_use.should == counter
      result[1].random.should == random
      result[1].crc.should == crc
      result[1].uid.should == uid
      result[0].should == check_otp
    end
  end
  
  describe "#public_id=" do
    it "should accepts modhex strings" do
      id = 'rfechkcdkjrflgrjlirdgivnedbbtcik'
      @key.public_id = id
      @key.public_id.should == id
      id = 'hehkhbhv'
      @key.public_id = id
      @key.public_id.should == id
    end
    
    it "should accepts only modhex strings" do
      id = @key.public_id
      lambda {@key.public_id = "pluto"}.should raise_error(YubiRuby::NoModHexEncodedError)
      @key.public_id.should == id
      lambda{@key.public_id = "pip!8"}.should raise_error(YubiRuby::NoModHexEncodedError)
      @key.public_id.should == id
    end
    
    it "should truncate longer string" do
      id = 'rfechkcdkjrflgrjlirdgivnedbbtcikrfechkcdkjrflgrjlirdgivnedbbtcik'
      (id.length > 32).should == true
      
      @key.public_id = id
      @key.public_id.should == id[0,32]
    end
  end
end
